function [x, y] = polar_mv(G, par)

  gam = par.prob_par.beta / par.prob_par.alpha;
  n1 = par.prob_par.m1;
  [n,t] = size(G);
  n2 = n - n1;
  
  G1 = G(1:n1,:);
  G2 = G(n1+1:end,:); 
  polar_par = par.polar_par;
  avg_eig = norm(G, 'fro')^2/n;
  lambda = 0;
  nu = 0;
  
  x = get_left_basis();  
  y = x'*G;
  y = y' / norm(y);
  x = x*par.prob_par.alpha;  
  
  function x = get_left_basis()
    
    % first try power iteration
    [x, min_eig] = power_iter();
    obj = norm(G'*x)^2;
    if min_eig / avg_eig > -polar_par.power_tol_eig
      if polar_par.verbose > 0
        fprintf('power method succeeded. Obj = %g\n', obj);  
      end
      return
    else
      if polar_par.verbose > 0
%         [V,D] = eig(G*G');
        fprintf('power method failed. Min_eig: %g, avg_eig: %g, obj: %g\n', ...
                  min_eig, avg_eig, obj);
      end 
    end

    % Do it rigorously.  First line search.
    [rho, f, iter, feval, msg] = pbm(@obj_ls, 1, 1e-8, inf, par.polar_par.pbm_opt);
    lambda = f / (1+gam^2*rho);   nu = lambda*rho;
    if polar_par.verbose > 0
      fprintf('obj by line search: %g, iter: %d, feval: %d, msg: %s\n', f, ...
              iter, feval, msg2str(msg, 'pbm'));
    end

    % Second try local optimization
    
    [x, obj, iter, feval, msg] = lbfgsb(x, -Inf(n,1), Inf(n,1), @obj_local, [], [], ...
                                 par.polar_par.lbfgs_opt);
    m1 = norm(x(1:n1));  m2 = norm(x(1+n1:end));
    x = [x(1:n1)/m1; x(1+n1:end)*gam/m2];
    f = norm(G'*x)^2;
    fprintf('obj by local search: %g, %g, iter: %d, feval: %d, msg: %s\n', ...
                  f, obj, iter, feval, msg2str(msg, 'lbfgsb'));  
    if obj < par.polar_par.local_tol
      if polar_par.verbose > 0
        fprintf('local search  succeeded. Obj: %g, %g, iter: %d, feval: %d, msg: %s\n', ...
                f, obj, iter, feval, msg2str(msg, 'lbfgsb')); 
      end
      return
    else
      fprintf('local search failed. Obj: %g, %g, iter: %d, feval: %d, msg: %s\n', ...
                f, obj, iter, feval, msg2str(msg, 'lbfgsb')); 
    end
    
    % Third use the brute force
    % First get the null space of M
    M = diag([lambda*ones(n1,1); nu*ones(n2,1)]) - G*G';
    GG = null_space(M, 1);
    GGx = GG(1:n1,:);   GGy = GG(1+n1:end,:);
    [P, S] = eig(gam^2*GGx'*GGx - GGy'*GGy);
    s = diag(S);
    k = length(s);
    [maxs, Imax] = max(s);      [mins, Imin] = min(s);
    if maxs - mins < 1e-7
      v = ones(k,1) * sqrt((1+gam^2)/k);
    else    
      Vmax = mins*(1+gam^2)/(mins - maxs);      
      Vmin = 1 + gam^2 - Vmax;
      v = zeros(k, 1);
      v(Imax) = sqrt(Vmax);       v(Imin) = sqrt(Vmin);    
    end
    x = GG*(P*v);
  end
  
  function f = aux_eig(x, rho)
    f = (1+rho*gam^2)*G1'*(G1*x) + (gam^2 + 1/rho)*G2'*(G2*x);
  end
  
  function [f, g] = obj_ls(rho)
    warning off;   [u, f] = laneig(@(x)aux_eig(x, rho), t, 1);  warning on;
    g1 = G1*u;  g2 = G2*u;
    g = g1'*g1*gam^2 - g2'*g2/rho^2;
  end

  function [x, min_eig] = power_iter()
    x1 = randn(n1,1);
    x2 = randn(n2,1);
    x = [x1/norm(x1)
         x2*(gam/norm(x2))];

    % power method iteration    
    for i = 1:polar_par.power_max_iter
      GGx = G*(G'*x);
      x1new = GGx(1:n1);
      x2new = GGx(1+n1:end);
      mu1 = norm(x1new);
      mu2 = norm(x2new);
      if mu1 < 1e-6 || mu2 < 1e-6,  break;      end
      xnew= [x1new/mu1
             x2new*(gam/mu2)];
      diff = norm(xnew - x);
%       fprintf('diff = %g\n', diff);
      if diff < par.polar_par.power_tol_diff, break; end
      x = xnew;
    end
    
    % Now check if power iteration finds the solution
    GGx = G*(G'*x);
    s1 = norm(GGx(1:n1)) / norm(x(1:n1));
    s2 = norm(GGx(1+n1:end)) / norm(x(1+n1:end));
    fhandle  = @(x)([s1*x(1:n1); s2*x(1+n1:end)] - G*(G'*x));
    warning off;    [~, min_eig] = laneig(fhandle , n1+n2, 1, 'AS');    warning on;
  end

  function [f, g] = obj_local(x)
    a = x(1:n1);  b = x(1+n1:end);
    G1a = G1'*a;   G2b = G2'*b;
    wgt = 10;
    norm_a = norm(a);   norm_b = norm(b);
    f = lambda*norm_a^2 + nu*norm_b^2 - G1a'*G1a - G2b'*G2b - 2 * G1a'*G2b ...
          + wgt*(norm_a - 1)^2/2 + wgt*(norm_b - gam)^2/2;
    if norm_a > 1e-6,  ga = a / norm_a;   else ga = zero(size(a));  end
    if norm_b > 1e-6,  gb = b / norm_b;   else gb = zero(size(b));  end
    g = [2*lambda*a - 2*G1*G1a - 2*G1*G2b + wgt*(norm_a-1)*ga;
         2*nu*b - 2*G2*G2b - 2*G2*G1a + wgt*(norm_b-gam)*gb];
  end

end